@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -42,6 +42,14 @@
 static void ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle);
 static gboolean totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl, const char *display_name);
 
+typedef gboolean (*PlaylistCallback) (TotemPlaylist *playlist, const char *mrl,
+		gpointer data);
+typedef gboolean (*ClearComparisonFunc) (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data);
+
+static void totem_playlist_clear_with_compare (TotemPlaylist *playlist,
+					       ClearComparisonFunc func,
+					       gconstpointer data);
+
 /* Callback function for GtkBuilder */
 void totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist);
 void totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist);
@@ -52,8 +60,6 @@ void playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *p
 void playlist_select_subtitle_action_callback (GtkAction *action, TotemPlaylist *playlist);
 void playlist_remove_action_callback (GtkAction *action, TotemPlaylist *playlist);
 
-typedef gboolean (*PlaylistCallback) (TotemPlaylist *playlist, const char *mrl,
-		gpointer data);
 
 typedef struct {
 	TotemPlaylist *playlist;
@@ -136,10 +142,8 @@ enum {
 	FILENAME_COL,
 	URI_COL,
 	TITLE_CUSTOM_COL,
-	CACHE_TITLE_COL,
-	CACHE_ARTIST_COL,
-	CACHE_ALBUM_COL,
 	SUBTITLE_URI_COL,
+	FILE_MONITOR_COL,
 	NUM_COLS
 };
 
@@ -267,6 +271,15 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 	GFile *file;
 	char *filename_for_display, *unescaped;
 
+	if (g_str_has_prefix (mrl, "dvd://") != FALSE) {
+		/* This is "Title 3", where title is a DVD title
+		 * Note: NOT a DVD chapter */
+		return g_strdup_printf (_("Title %d"), (int) g_strtod (mrl + 6, NULL));
+	} else if (g_str_has_prefix (mrl, "dvb://") != FALSE) {
+		/* This is "BBC ONE(BBC)" for "dvb://BBC ONE(BBC)" */
+		return g_strdup (mrl + 6);
+	}
+
 	file = g_file_new_for_uri (mrl);
 	unescaped = g_file_get_basename (file);
 	g_object_unref (file);
@@ -448,19 +461,40 @@ drop_cb (GtkWidget        *widget,
 void
 playlist_select_subtitle_action_callback (GtkAction *action, TotemPlaylist *playlist)
 {
-	char *subtitle;
+	char *subtitle, *current, *path;
 	GList *l;
+	GFile *file, *dir;
 	TotemPlaylistStatus playing;
 	GtkTreeIter iter;
 
-	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), NULL);
+	l = gtk_tree_selection_get_selected_rows (playlist->priv->selection, NULL);
+	gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
+	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (l);
+
+	/* Look for the directory of the current movie */
+	gtk_tree_model_get (playlist->priv->model, &iter,
+			    FILENAME_COL, &current,
+			    -1);
+
+	if (current == NULL)
+		return;
+
+	path = NULL;
+	file = g_file_new_for_commandline_arg (current);
+	dir = g_file_get_parent (file);
+	g_object_unref (file);
+	if (dir != NULL) {
+		path = g_file_get_path (dir);
+		g_object_unref (dir);
+	}
+
+	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), path);
+	g_free (path);
 
 	if (subtitle == NULL)
 		return;
 
-	l = gtk_tree_selection_get_selected_rows (playlist->priv->selection, NULL);
-	gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
-
 	gtk_tree_model_get (playlist->priv->model, &iter,
 			    PLAYING_COL, &playing,
 			    -1);
@@ -482,16 +516,15 @@ void
 playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *playlist)
 {
 	GList *l;
-	GtkTreePath *path;
 	GtkClipboard *clip;
 	char *url;
 	GtkTreeIter iter;
 
 	l = gtk_tree_selection_get_selected_rows (playlist->priv->selection,
 			NULL);
-	path = l->data;
-
-	gtk_tree_model_get_iter (playlist->priv->model, &iter, path);
+	gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
+	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (l);
 
 	gtk_tree_model_get (playlist->priv->model,
 			&iter,
@@ -507,8 +540,6 @@ playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *playli
 	gtk_clipboard_set_text (clip, url, -1);
 	g_free (url);
 
-	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
-	g_list_free (l);
 }
 
 static gboolean
@@ -1297,26 +1328,8 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 static void
 init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 {
-	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 
-	/* the model */
-	model = GTK_TREE_MODEL (gtk_list_store_new (NUM_COLS,
-				G_TYPE_INT,
-				G_TYPE_STRING,
-				G_TYPE_STRING,
-				G_TYPE_BOOLEAN,
-				G_TYPE_STRING,
-				G_TYPE_STRING,
-				G_TYPE_STRING,
-				G_TYPE_STRING));
-
-	/* the treeview */
-	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
-	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
-	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
-	g_object_unref (G_OBJECT (model));
-
 	init_columns (GTK_TREE_VIEW (treeview), playlist);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
@@ -1519,6 +1532,43 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 	totem_playlist_add_one_mrl (playlist, uri, title);
 }
 
+static gboolean
+totem_playlist_compare_with_monitor (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data)
+{
+	GFileMonitor *monitor = (GFileMonitor *) data;
+	GFileMonitor *_monitor;
+	gboolean retval = FALSE;
+
+	gtk_tree_model_get (playlist->priv->model, iter,
+			    FILE_MONITOR_COL, &_monitor, -1);
+
+	if (_monitor == monitor)
+		retval = TRUE;
+
+	if (_monitor != NULL)
+		g_object_unref (_monitor);
+
+	return retval;
+}
+
+static void
+totem_playlist_file_changed (GFileMonitor *monitor,
+			     GFile *file,
+			     GFile *other_file,
+			     GFileMonitorEvent event_type,
+			     TotemPlaylist *playlist)
+{
+	if (event_type == G_FILE_MONITOR_EVENT_DELETED) {
+		char *uri;
+
+		uri = g_file_get_uri (file);
+		totem_playlist_clear_with_compare (playlist,
+						   (ClearComparisonFunc) totem_playlist_compare_with_monitor,
+						   monitor);
+		g_free (uri);
+	}
+}
+
 static void
 totem_playlist_dispose (GObject *object)
 {
@@ -1603,8 +1653,6 @@ totem_playlist_init (TotemPlaylist *playlist)
 	init_config (playlist);
 
 	gtk_widget_show_all (GTK_WIDGET (playlist));
-
-
 }
 
 GtkWidget*
@@ -1629,6 +1677,8 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	GtkTreeIter iter;
 	char *filename_for_display, *uri;
 	GtkTreeRowReference *ref;
+	GFileMonitor *monitor;
+	GFile *file;
 	int pos;
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
@@ -1655,11 +1705,28 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	}
 
 	store = GTK_LIST_STORE (playlist->priv->model);
+
+	/* Get the file monitor */
+	file = g_file_new_for_uri (uri ? uri : mrl);
+	if (g_file_is_native (file) != FALSE) {
+		monitor = g_file_monitor_file (file,
+					       G_FILE_MONITOR_NONE,
+					       NULL,
+					       NULL);
+		g_signal_connect (G_OBJECT (monitor),
+				  "changed",
+				  G_CALLBACK (totem_playlist_file_changed),
+				  playlist);
+	} else {
+		monitor = NULL;
+	}
+
 	gtk_list_store_insert_with_values (store, &iter, pos,
 			PLAYING_COL, TOTEM_PLAYLIST_STATUS_NONE,
 			FILENAME_COL, filename_for_display,
 			URI_COL, uri ? uri : mrl,
 			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
+			FILE_MONITOR_COL, monitor,
 			-1);
 
 	g_signal_emit (playlist,
@@ -1765,22 +1832,24 @@ totem_playlist_clear (TotemPlaylist *playlist)
 }
 
 static void
-totem_playlist_clear_with_compare (TotemPlaylist *playlist, GCompareFunc func,
-		gconstpointer data)
+totem_playlist_clear_with_compare (TotemPlaylist *playlist,
+				   ClearComparisonFunc func,
+				   gconstpointer data)
 {
 	GList *list = NULL, *l;
 	guint num_items, i;
-	gboolean has_items;
+	gboolean has_items, current_removed;
 
 	num_items = PL_LEN;
 	if (num_items == 0)
 		return;
 
+	current_removed = FALSE;
+
 	for (i = 0; i < num_items; i++)
 	{
 		GtkTreeIter iter;
 		char *index;
-		char *mrl;
 
 		index = g_strdup_printf ("%d", i);
 		if (gtk_tree_model_get_iter_from_string
@@ -1792,10 +1861,7 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist, GCompareFunc func,
 		}
 		g_free (index);
 
-		gtk_tree_model_get (playlist->priv->model, &iter,
-				URI_COL, &mrl, -1);
-
-		if ((* func) (mrl, data) != FALSE)
+		if ((* func) (playlist, &iter, data) != FALSE)
 		{
 			GtkTreePath *path;
 			GtkTreeRowReference *ref;
@@ -1806,8 +1872,6 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist, GCompareFunc func,
 			list = g_list_prepend (list, ref);
 			gtk_tree_path_free (path);
 		}
-
-		g_free (mrl);
 	}
 
 	has_items = (list != NULL);
@@ -1819,6 +1883,8 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist, GCompareFunc func,
 
 		path = gtk_tree_row_reference_get_path (l->data);
 		gtk_tree_model_get_iter (playlist->priv->model, &iter, path);
+		if (gtk_tree_path_compare (path, playlist->priv->current) == 0)
+			current_removed = TRUE;
 
 		totem_playlist_emit_item_removed (playlist, &iter);
 		gtk_list_store_remove (GTK_LIST_STORE (playlist->priv->model), &iter);
@@ -1827,7 +1893,7 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist, GCompareFunc func,
 	}
 	g_list_free (list);
 
-	if (has_items != FALSE) {
+	if (has_items != FALSE && current_removed != FALSE) {
 		playlist->priv->current_shuffled = -1;
 
 		ensure_shuffled (playlist, playlist->priv->shuffle);
@@ -1836,19 +1902,27 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist, GCompareFunc func,
 		g_signal_emit (G_OBJECT (playlist),
 				totem_playlist_table_signals[CURRENT_REMOVED],
 				0, NULL);
+	} else if (has_items != FALSE) {
+		ensure_shuffled (playlist, playlist->priv->shuffle);
+		g_signal_emit (G_OBJECT (playlist),
+			       totem_playlist_table_signals[CHANGED], 0,
+			       NULL);
 	}
 }
 
-static int
-totem_playlist_compare_with_mount (gpointer a, gpointer b)
+static gboolean
+totem_playlist_compare_with_mount (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data)
 {
-	GMount *clear_mount = (GMount *) b;
-	const char *mrl = (const char *) a;
+	GMount *clear_mount = (GMount *) data;
+	char *mrl;
 
 	GMount *mount;
 	gboolean retval = FALSE;
 
+	gtk_tree_model_get (playlist->priv->model, iter,
+			    URI_COL, &mrl, -1);
 	mount = totem_get_mount_for_media (mrl);
+	g_free (mrl);
 
 	if (mount == clear_mount)
 		retval = TRUE;
@@ -1864,7 +1938,7 @@ totem_playlist_clear_with_g_mount (TotemPlaylist *playlist,
 				   GMount *mount)
 {
 	totem_playlist_clear_with_compare (playlist,
-					   (GCompareFunc) totem_playlist_compare_with_mount,
+					   (ClearComparisonFunc) totem_playlist_compare_with_mount,
 					   mount);
 }
 
@@ -1948,34 +2022,6 @@ totem_playlist_get_title (TotemPlaylist *playlist, guint index)
 	return title;
 }
 
-gboolean
-totem_playlist_get_current_metadata (TotemPlaylist *playlist,
-				     char         **artist,
-				     char         **title,
-				     char         **album)
-{
-	GtkTreeIter iter;
-
-        g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
-
-	if (update_current_from_playlist (playlist) == FALSE)
-		return FALSE;
-
-	gtk_tree_model_get_iter (playlist->priv->model,
-				 &iter,
-				 playlist->priv->current);
-
-	*artist = NULL;
-	gtk_tree_model_get (playlist->priv->model,
-			    &iter,
-			    CACHE_ARTIST_COL, artist,
-			    CACHE_TITLE_COL, title,
-			    CACHE_ALBUM_COL, album,
-			    -1);
-
-	return (*artist != NULL);
-}
-
 gboolean
 totem_playlist_has_previous_mrl (TotemPlaylist *playlist)
 {
